load("MPLACE");
load("TRANS");
load("CHAIN");
load("FILTERBANK");

sample_rate = 44100;
channels = 2;
bufsize = 512;

print_on();
srand(2323423);

outfile = s_arg(0);

if (outfile != 0) {
    set_option("audio_off");
	rtsetparams(sample_rate, channels, bufsize);
    rtoutput(outfile, "24");
}
else {
	rtsetparams(sample_rate, channels, bufsize);
}

rtinput("sounds/ice.wav");


bus_config("TRANS3", "in 0", "chain 0 out");
bus_config("MPLACE", "chain 0 in", "chain 1-4 out");
bus_config("CHAIN", "aux 0-3 out");
bus_config("RVB", "aux 0-3 in", "out 0-1");

dist_front=45
dist_right=47
dist_rear=-35
dist_left=-66
height=17
rvbtime=0.5
abs_fac=9

space(dist_front,dist_right,dist_rear,dist_left,height,abs_fac,rvbtime)

mike_dist = 6;
mikes(45.0, 0.5);

// This table lets me pick offsets in seconds (first number) into the recorded audio file.
// The second number is the length in seconds of the sound at that offset.  This lets me treat
// a single long audio file as a set of short recordings.

mixes = {
	{ 3.425,		0.678 },	// short
	{ 7.028,		0.397 },	// short
	{ 10.497,	0.497 },	// short
	{ 36.381,	0.877 },
	{ 38.953,	0.547 },
	{ 41.013,	0.506 },
	{ 50.552,	0.585 },
	{ 51.884,	0.607 },
	
	{ 13.87,		3.026 },	// long
	{ 18.019,	2.476 },
	{ 21.937,	3.413 },
	{ 25.782,	0.626 },	// 0.212 lead in
	{ 27.145,	0.964 },	// 0.207 lead in
	{ 28.80,		0.980 },
	{ 43.661,	1.002 },
	{ 45.614,	1.182 },
	{ 47.952,	1.452 },
	{ 54.426,	1.294 },
	{ 56.836,	1.177 }
}

// Set up the first beat pattern for the ice ostinato

whole = 1;
half = 0.5;
quarter = 0.25;
eigth = 0.125;

beat1 = { quarter, quarter, eigth, eigth, eigth, eigth, eigth, quarter, quarter, eigth, eigth, eigth };

// Set up the left-right position pattern for same

right = dist_right * 0.75;
left = dist_left * 0.75;

positions1 = { right, left, right, right, left, left, left, right, left, left, right, right };

kMaxShortIndex = 7;
kMinLongIndex = 8;
kMaxLongIndex = 10;

amp1 = 75;
gTranspose = 0.0;
gSkew = 0.0;
gYpos = dist_front * 0.25;

// A utility which lets me specify how far I want to read into a table as a fraction from 0.0 to 1.0

float sampleline(handle table, float percent)
{
        tablelength = tablelen(table)
        return samptable(table, min(1.0, percent) * tablelength);
}

// This sets the global tempo for the score.  First number is start time, second is beats per second

tempo(0, 70);

piecelength = 60 * 3;			// This is in seconds...
piecebeats = tb(piecelength);	// and tb() converts that into beats

// This table is used to modify the "x" coordinate of each sound (I am using Cartesian coordinates)
gXModTable = maketable("line", 10000, 0, 1, 1, .5, 2, 1, 3, .25, 4, .5, 5, .125, 10, 1);

float runshortbeats(float start, list beats, list positions, float minIndex, float maxIndex, float amp)
{
	numbeats = len(beats);
	offset = 0;
	// sampleline() walks the above table exactly once over 'piecebeats' beats
	xscale = sampleline(gXModTable, start/piecebeats);
	for (n = 0; n < numbeats; n += 1) {
		mixindex = trand(minIndex, maxIndex);
		entry = mixes[mixindex];
		inskip = entry[0];
		inpitch = gTranspose;
		dur = translen(entry[1], inpitch);
		setexp(0,1,10,1,30,0.001);
		trans = makeinstrument("TRANS3",0,inskip,dur,amp,inpitch,0);
		x = positions[n] * xscale;
		y = gYpos;
		setexp(0,1,1,1);
		mplace = makeinstrument("MPLACE", 0, 0, dur, 1.0, x, y, -mike_dist, 0);
		CHAIN(bt(start+offset) + gSkew + irand(-0.005, 0.005), dur, 2, trans, mplace);
		offset += beats[n];
	}
	return offset;
}

start = 0.005;

while (start < piecebeats) {
	if (start > piecebeats - 20) {
		amp1 *= 0.5;
		printf("amp1 now %f\n", amp1);
	}
	start += runshortbeats(start, beat1, positions1, 0, kMaxShortIndex, amp1);
}

// Set up the second beat pattern for the ice ostinato

beat2 = { eigth, quarter, quarter, eigth, quarter, eigth, eigth, quarter };
right = dist_right * 0.3;
left = dist_left * 0.3;
positions2 = { right, right, left, right, left, left, right, left };

start2 = 4;

amp2 = 40;
gTranspose = -0.07;
gSkew = -0.01;
gYpos = dist_front * 0.75;

while (start2 < piecebeats) {
	if (start2 > piecebeats - 20) {
		amp2 *= 0.5;
	}
	start2 += runshortbeats(start2, beat2, positions2, 0, kMaxShortIndex-5, amp2);
}

// Set up the third beat pattern for the ice ostinato

beat3 = { eigth, eigth, quarter, eigth };
positions3 = { -5, 5, -5, -5 };
gYpos = 15;
amp3 = 280;
gTranspose = 1.09;
gSkew = 0;
start3 = 16;

while (start3 < piecebeats) {
	if (start3 > piecebeats - 20) {
		amp3 *= 0.25;
	}
	runshortbeats(start3, beat3, positions3, 11, 13, amp3);
	start3 += 16;
}

// Transposing samples down more than 4 octaves (original_pitch/16) to produce "gong"
// Note that these play every 17 seconds.

beat4 = { 0 }	// This makes runshortbeats() play exactly one "bong"
positions4 = { dist_right * 0.5 };
gYpos = 40;
amp4 = 55;
gTranspose = -4.03;
gSkew = -0.1;
start4 = 17;
while (start4 < piecebeats) {
	if (start4 > piecebeats - 20) {
		amp4 *= 0.25;
	}
	runshortbeats(start4, beat4, positions4, 0, 2, amp4);
	start4 += 17;
}

// Setup for foreground instrument - sort of like a vibraphone due to the use of the FILTERBANK instrument

bus_config("FILTERBANK", "in 0", "chain 0 out");
bus_config("MPLACE", "chain 0 in", "chain 1-4 out");
bus_config("CHAIN", "aux 0-3 out");
bus_config("RVB", "aux 0-3 in", "out 0-1");

float filterplace(float start, float amp)
{
	// here we pick which portion of the recorded ice sameple we play
	mixindex = pickrand(8, 9, 10, 14, 15, 16, 17);
	entry = mixes[mixindex];
	inskip = entry[0];
	ringdur = 10;
	bw = 0.001;
	dur = entry[1];

	// The notes we chose between, used as center frequencies for the filter
	keys = { 8.02, 8.04, 8.06, 8.07, 8.09, 8.11, 9.01, 9.02, 9.04, 9.06, 9.07, 9.09, 9.11 };

	filter = makeinstrument("FILTERBANK", 0, inskip, dur, 1000, ringdur, 0, pan=0,
		  pickrand(keys), bw, g=1,
		  pickrand(keys), bw, g=1,
		  pickrand(keys), bw, g=1,
		  pickrand(keys), bw, g=1,
		  pickrand(keys), bw, g=1)

	fdur = dur + ringdur;

	xpos = irand(dist_left, dist_right);
	ypos = dist_front - 2;
	dmove = makeinstrument("MPLACE",0,0,fdur,amp,12,ypos,-mike_dist,0);
	CHAIN(bt(start), fdur, 2, filter, dmove);
	return 0;
}

chordstart = 11;
amp5 = 1.2;

while (chordstart < piecebeats - 10) {
	if (chordstart > piecebeats - 25) {
		amp5 *= 0.25;
	}
	filterplace(chordstart, amp5);
	chordstart += pickrand(2, 3, 3, 5, 5, 7, 7, 9, 11);
}

rvblevel = maketable("line", 10000, 0, 0.1, piecelength-20, 0.1, piecelength, 1);
RVB(0, 0, bt(max(chordstart, start2, start)) + rvbtime, rvblevel);
