sample_rate = 44100;
channels = 2;
bufsize = 512;

print_on();
srand(2323423);

outfile = s_arg(0);

if (outfile != 0) {
    set_option("audio_off");
	rtsetparams(sample_rate, channels, bufsize);
    rtoutput(outfile, "24");
}
else {
	rtsetparams(sample_rate, channels, bufsize);
}

load("MPLACE");
load("TRANS");
load("CHAIN");

rtinput("/Users/dscott/disquiet/disquiet0262-icecode/sounds/ice2016.wav");

bus_config("TRANS3", "in 0", "chain 0 out");
bus_config("MPLACE", "chain 0 in", "chain 1-4 out");
bus_config("CHAIN", "aux 0-3 out");
bus_config("RVB", "aux 0-3 in", "out 0-1");

dist_front=95
dist_right=87
dist_rear=-85
dist_left=-96
height=77
rvbtime=4.5
abs_fac=5
rvblevel = 0.05;

space(dist_front,dist_right,dist_rear,dist_left,height,abs_fac,rvbtime)

mike_dist = 6;
mikes(45.0, 0.5);

// Same mix table I used in the previous year's composition

mixes = {
	{ 3.425,	0.678 },	// short
	{ 7.028,	0.397 },	// short
	{ 10.497,	0.497 },	// short
	{ 36.381,	0.877 },
	{ 38.953,	0.547 },
	{ 41.013,	0.506 },
	{ 50.552,	0.585 },
	{ 51.884,	0.607 },
	
	{ 13.87,	3.026 },	// long
	{ 18.019,	2.476 },
	{ 21.937,	3.413 },
	{ 25.782,	0.626 },	// 0.212 lead in
	{ 27.145,	0.964 },	// 0.207 lead in
	{ 28.80,	0.980 },
	{ 43.661,	1.002 },
	{ 45.614,	1.182 },
	{ 47.952,	1.452 },
	{ 54.426,	1.294 },
	{ 56.836,	1.177 }
}

// Function to run sounds through TRANS3 with an exponential gain decay, then on through MPLACE

float playEntry(float index, float start, float amp, float pitch, float x, float y)
{
	entry = mixes[index];
	inskip = entry[0];
	dur = translen(entry[1], pitch);
	setexp(0,1,10,1,30,0.001);
	trans = makeinstrument("TRANS3",0,inskip,dur,amp,pitch,0);
	setexp(0,1,1,1);
	mplace = makeinstrument("MPLACE", 0, 0, dur, 1.0, x, y, -mike_dist, 0);
	CHAIN(start, dur, 2, trans, mplace);
	return start + dur;
}

// These divide up the sample space into short and long snippets
kMinLongIndex = 8;
kMaxLongIndex = 18;

// Set the boundaries of the sound locations based on the dimensions of the room
kMaxRight = dist_right - 4;
kMaxLeft = dist_left + 4;
kMaxFront = dist_front - 4;
kMaxRear = dist_rear + 4;

// Short events

incr = 0.1;
start = 0.1;
last = 0;
gain = 15;
jitter = 0.001;

while (last < 120) {
	// pick from "shorter" sounds
	index = trand(0, kMinLongIndex-1);
	// constrained random transposition over range 0.0 to 0.10
	pitch = 0.01 * pickrand(0, 1, 3, 5, 6, 8, 9, 10);
	// constrained random locations in room
	x = pickrand(-6, -3, 3, 6);
	y = irand(kMaxFront/8, kMaxFront/4);
	// Leave a hole statistically every 6 out of 7 times
	if (chance(6, 7)) {
		// How many sound events during this segment of time?
		density = trand(0, 7);
		// How far off-center can our event timing be?
		dspread = irand(0.001, 0.004);
		for (i = 0; i < density; i += 1) {
			// Small constrained random variations in pitch
			spread = irand(0.001, 0.004);
			last = max(playEntry(index, start+irand(-dspread, dspread), gain, pitch+spread, x + irand(-1, 1), y + irand(-1, 1)), last);
		}
	}
	// offset to next segment, with small jitter
	start += incr + irand(-jitter, jitter);
}

// Now we do something similar for the longer sample segments

last = 0;
start = 4;
gain = 100;
incr = 0.3;
jitter = 0.01;

xspread = 0;
yspread = 0;
kMaxXSpread = kMaxRight/4;
kMaxYSpread = kMaxFront/15;

while (last < 120) {
	index = trand(kMinLongIndex, kMaxLongIndex);
	pitch = 1.00 + (0.01 * pickrand(0, 1, 3, 5, 6, 8, 9, 10));
	spread = irand(0.0005, 0.002);
	x = irand(kMaxLeft/2, kMaxRight/2);
	y = irand(20, kMaxFront/2);

	if (chance(3, 4)) {
		last = max(playEntry(index, start, gain, pitch+spread, x + irand(-xspread, xspread), y + irand(-yspread, yspread)), last);
		last = max(playEntry(index, start, gain, pitch-spread, x + irand(-xspread, xspread), y + irand(-yspread, yspread)), last);
	}

	start += incr + irand(-jitter, jitter);
	xspread = (last/180) * kMaxXSpread;
	yspread = (last/180) * kMaxYSpread;
}

// Lastly, we add a layer of down-transposed samples, spaced 2-4 seconds apart

start = 5;
last = 0;
gain = 120;

while (start < 100) {
	index = trand(kMinLongIndex, kMaxLongIndex);
	start = start + irand(2.1, 4.5);
	pitch = pchoct(irand(-2.9, -1.4));
	spread = irand(0.001, 0.004);
	x = irand(kMaxLeft, kMaxRight);
	y = irand(10, kMaxFront);

	last = max(playEntry(index, start, gain, pitch+spread, x + irand(-2, 2), y + irand(-2, 2)), last);
	last = max(playEntry(index, start, gain, pitch-spread, x + irand(-2, 2), y + irand(-2, 2)), last);
}


RVB(0, 0, last+rvbtime, rvblevel);
